@@ -559,11 +559,11 @@ static void do_supdup_read(Supdup *supdup, const char *buf, size_t len)
     strbuf_free(outbuf);
 }
 
-static void supdup_log(Plug *plug, PlugLogType type, SockAddr *addr, int port,
-                       const char *error_msg, int error_code)
+static void supdup_log(Plug *plug, Socket *s, PlugLogType type, SockAddr *addr,
+                       int port, const char *error_msg, int error_code)
 {
     Supdup *supdup = container_of(plug, Supdup, plug);
-    backend_socket_log(supdup->seat, supdup->logctx, type, addr, port,
+    backend_socket_log(supdup->seat, supdup->logctx, s, type, addr, port,
                        error_msg, error_code,
                        supdup->conf, supdup->socket_connected);
     if (type == PLUGLOG_CONNECT_SUCCESS) {
