@@ -80,7 +80,7 @@ int platform_default_i(const char *name, int def)
 
 FontSpec *platform_default_fontspec(const char *name)
 {
-    return fontspec_new("");
+    return fontspec_new_default();
 }
 
 Filename *platform_default_filename(const char *name)
@@ -478,8 +478,8 @@ static Plug *server_conn_plug(
         &inst->ap, &inst->logpolicy, &unix_live_sftpserver_vt);
 }
 
-static void server_log(Plug *plug, PlugLogType type, SockAddr *addr, int port,
-                       const char *error_msg, int error_code)
+static void server_log(Plug *plug, Socket *s, PlugLogType type, SockAddr *addr,
+                       int port, const char *error_msg, int error_code)
 {
     log_to_stderr((unsigned)-1, error_msg);
 }
@@ -514,13 +514,13 @@ static int server_accepting(Plug *p, accept_fn_t constructor, accept_ctx_t ctx)
     if ((err = sk_socket_error(s)) != NULL)
         return 1;
 
-    SocketPeerInfo *pi = sk_peer_info(s);
+    SocketEndpointInfo *pi = sk_peer_info(s);
 
     if (pi->addressfamily != ADDRTYPE_LOCAL && !sk_peer_trusted(s)) {
         fprintf(stderr, "rejected connection to serv#%u "
                 "from %s (untrustworthy peer)\n",
                 cfg->config_id, pi->log_text);
-        sk_free_peer_info(pi);
+        sk_free_endpoint_info(pi);
         sk_close(s);
         next_id = old_next_id;
         return 1;
@@ -530,7 +530,7 @@ static int server_accepting(Plug *p, accept_fn_t constructor, accept_ctx_t ctx)
                           cfg->config_id, pi->log_text);
     log_to_stderr(inst->id, msg);
     sfree(msg);
-    sk_free_peer_info(pi);
+    sk_free_endpoint_info(pi);
 
     sk_set_frozen(s, false);
     ssh_server_start(plug, s);
@@ -924,6 +924,8 @@ int main(int argc, char **argv)
             ci->ssc.stunt_allow_trivial_ki_auth = true;
         } else if (!strcmp(arg, "--return-success-to-pubkey-offer")) {
             ci->ssc.stunt_return_success_to_pubkey_offer = true;
+        } else if (!strcmp(arg, "--close-after-banner")) {
+            ci->ssc.stunt_close_after_banner = true;
         } else {
             fprintf(stderr, "%s: unrecognised option '%s'\n", appname, arg);
             exit(1);
