@@ -393,13 +393,11 @@ char *staticwrap(struct ctlpos *cp, HWND hwnd, const char *text, int *lines)
     INT *pwidths, nfit;
     SIZE size;
     const char *p;
-    char *ret, *q;
     RECT r;
     HFONT oldfont, newfont;
 
-    ret = snewn(1+strlen(text), char);
+    strbuf *sb = strbuf_new();
     p = text;
-    q = ret;
     pwidths = snewn(1+strlen(text), INT);
 
     /*
@@ -432,7 +430,7 @@ char *staticwrap(struct ctlpos *cp, HWND hwnd, const char *text, int *lines)
              * Either way, we stop wrapping, copy the remainder of
              * the input string unchanged to the output, and leave.
              */
-            strcpy(q, p);
+            put_datapl(sb, ptrlen_from_asciz(p));
             break;
         }
 
@@ -449,9 +447,8 @@ char *staticwrap(struct ctlpos *cp, HWND hwnd, const char *text, int *lines)
             }
         }
 
-        strncpy(q, p, nfit);
-        q[nfit] = '\n';
-        q += nfit+1;
+        put_data(sb, p, nfit);
+        put_byte(sb, '\n');
 
         p += nfit;
         while (*p && isspace((unsigned char)*p))
@@ -467,7 +464,7 @@ char *staticwrap(struct ctlpos *cp, HWND hwnd, const char *text, int *lines)
 
     sfree(pwidths);
 
-    return ret;
+    return strbuf_to_str(sb);
 }
 
 /*
@@ -1181,30 +1178,24 @@ void progressbar(struct ctlpos *cp, int id)
  */
 static char *shortcut_escape(const char *text, char shortcut)
 {
-    char *ret;
-    char const *p;
-    char *q;
-
     if (!text)
         return NULL;                   /* sfree won't choke on this */
 
-    ret = snewn(2*strlen(text)+1, char);   /* size potentially doubles! */
+    strbuf *sb = strbuf_new();
     shortcut = tolower((unsigned char)shortcut);
 
-    p = text;
-    q = ret;
+    const char *p = text;
     while (*p) {
         if (shortcut != NO_SHORTCUT &&
             tolower((unsigned char)*p) == shortcut) {
-            *q++ = '&';
+            put_byte(sb, '&');
             shortcut = NO_SHORTCUT;    /* stop it happening twice */
         } else if (*p == '&') {
-            *q++ = '&';
+            put_byte(sb, '&');
         }
-        *q++ = *p++;
+        put_byte(sb, *p++);
     }
-    *q = '\0';
-    return ret;
+    return strbuf_to_str(sb);
 }
 
 void winctrl_add_shortcuts(struct dlgparam *dp, struct winctrl *c)
@@ -1692,7 +1683,7 @@ void winctrl_layout(struct dlgparam *dp, struct winctrls *wc,
             shortcuts[nshortcuts++] = ctrl->fontselect.shortcut;
             statictext(&pos, escaped, 1, base_id);
             staticbtn(&pos, "", base_id+1, "Change...", base_id+2);
-            data = fontspec_new("", false, 0, 0);
+            data = fontspec_new_default();
             sfree(escaped);
             break;
           default:
@@ -1997,32 +1988,36 @@ bool winctrl_handle_command(struct dlgparam *dp, UINT msg,
             (msg == WM_COMMAND &&
              (HIWORD(wParam) == BN_CLICKED ||
               HIWORD(wParam) == BN_DOUBLECLICKED))) {
-            OPENFILENAME of;
-            char filename[FILENAME_MAX];
+            OPENFILENAMEW of;
+            wchar_t filename[FILENAME_MAX];
+
+            wchar_t *title_to_free = NULL;
 
             memset(&of, 0, sizeof(of));
             of.hwndOwner = dp->hwnd;
             if (ctrl->fileselect.filter)
                 of.lpstrFilter = ctrl->fileselect.filter;
             else
-                of.lpstrFilter = "All Files (*.*)\0*\0\0\0";
+                of.lpstrFilter = L"All Files (*.*)\0*\0\0\0";
             of.lpstrCustomFilter = NULL;
             of.nFilterIndex = 1;
             of.lpstrFile = filename;
             if (!ctrl->fileselect.just_button) {
-                GetDlgItemText(dp->hwnd, c->base_id+1,
-                               filename, lenof(filename));
-                filename[lenof(filename)-1] = '\0';
+                GetDlgItemTextW(dp->hwnd, c->base_id+1,
+                                filename, lenof(filename));
+                filename[lenof(filename)-1] = L'\0';
             } else {
-                *filename = '\0';
+                *filename = L'\0';
             }
             of.nMaxFile = lenof(filename);
             of.lpstrFileTitle = NULL;
-            of.lpstrTitle = ctrl->fileselect.title;
+            of.lpstrTitle = title_to_free = dup_mb_to_wc(
+                DEFAULT_CODEPAGE, ctrl->fileselect.title);
             of.Flags = 0;
-            if (request_file(NULL, &of, false, ctrl->fileselect.for_writing)) {
+            if (request_file_w(NULL, &of, false,
+                               ctrl->fileselect.for_writing)) {
                 if (!ctrl->fileselect.just_button) {
-                    SetDlgItemText(dp->hwnd, c->base_id + 1, filename);
+                    SetDlgItemTextW(dp->hwnd, c->base_id + 1, filename);
                     ctrl->handler(ctrl, dp, dp->data, EVENT_VALCHANGE);
                 } else {
                     assert(!c->data);
@@ -2031,6 +2026,8 @@ bool winctrl_handle_command(struct dlgparam *dp, UINT msg,
                     c->data = NULL;
                 }
             }
+
+            sfree(title_to_free);
         }
         break;
       case CTRL_FONTSELECT:
@@ -2213,6 +2210,15 @@ void dlg_editbox_set(dlgcontrol *ctrl, dlgparam *dp, char const *text)
     SetDlgItemText(dp->hwnd, c->base_id+1, text);
 }
 
+void dlg_editbox_set_utf8(dlgcontrol *ctrl, dlgparam *dp, char const *text)
+{
+    struct winctrl *c = dlg_findbyctrl(dp, ctrl);
+    assert(c && c->ctrl->type == CTRL_EDITBOX);
+    wchar_t *wtext = dup_mb_to_wc(CP_UTF8, text);
+    SetDlgItemTextW(dp->hwnd, c->base_id+1, wtext);
+    sfree(wtext);
+}
+
 char *dlg_editbox_get(dlgcontrol *ctrl, dlgparam *dp)
 {
     struct winctrl *c = dlg_findbyctrl(dp, ctrl);
@@ -2220,6 +2226,16 @@ char *dlg_editbox_get(dlgcontrol *ctrl, dlgparam *dp)
     return GetDlgItemText_alloc(dp->hwnd, c->base_id+1);
 }
 
+char *dlg_editbox_get_utf8(dlgcontrol *ctrl, dlgparam *dp)
+{
+    struct winctrl *c = dlg_findbyctrl(dp, ctrl);
+    assert(c && c->ctrl->type == CTRL_EDITBOX);
+    wchar_t *wtext = GetDlgItemTextW_alloc(dp->hwnd, c->base_id+1);
+    char *text = dup_wc_to_mb(CP_UTF8, wtext, "");
+    sfree(wtext);
+    return text;
+}
+
 void dlg_editbox_select_range(dlgcontrol *ctrl, dlgparam *dp,
                               size_t start, size_t len)
 {
@@ -2403,19 +2419,17 @@ void dlg_filesel_set(dlgcontrol *ctrl, dlgparam *dp, Filename *fn)
     assert(c);
     assert(c->ctrl->type == CTRL_FILESELECT);
     assert(!c->ctrl->fileselect.just_button);
-    SetDlgItemText(dp->hwnd, c->base_id+1, fn->path);
+    SetDlgItemTextW(dp->hwnd, c->base_id+1, fn->wpath);
 }
 
 Filename *dlg_filesel_get(dlgcontrol *ctrl, dlgparam *dp)
 {
     struct winctrl *c = dlg_findbyctrl(dp, ctrl);
-    char *tmp;
-    Filename *ret;
     assert(c);
     assert(c->ctrl->type == CTRL_FILESELECT);
     if (!c->ctrl->fileselect.just_button) {
-        tmp = GetDlgItemText_alloc(dp->hwnd, c->base_id+1);
-        ret = filename_from_str(tmp);
+        wchar_t *tmp = GetDlgItemTextW_alloc(dp->hwnd, c->base_id+1);
+        Filename *ret = filename_from_wstr(tmp);
         sfree(tmp);
         return ret;
     } else {
